@@ -140,8 +140,12 @@ class _ConfoundsCorrelationPlotInputSpec(BaseInterfaceInputSpec):
         "which all correlation magnitudes "
         "should be ranked and plotted",
     )
+    columns = traits.List(
+        traits.Str,
+        desc="Filter out all regressors not found in this list."
+    )
     max_dim = traits.Int(
-        70,
+        20,
         usedefault=True,
         desc="Maximum number of regressors to include in "
         "plot. Regressors with highest magnitude of "
@@ -172,8 +176,9 @@ class ConfoundsCorrelationPlot(SimpleInterface):
             self._results["out_file"] = self.inputs.out_file
         confounds_correlation_plot(
             confounds_file=self.inputs.confounds_file,
+            columns=self.inputs.columns if isdefined(self.inputs.columns) else None,
+            max_dim=self.inputs.max_dim,
             output_file=self._results["out_file"],
             reference=self.inputs.reference_column,
-            max_dim=self.inputs.max_dim,
         )
         return runtime
